package org.chulk.codegen;

import java.util.List;
import java.util.Queue;

import org.chulk.codegen.command.construct.ModelValue;
import org.chulk.codegen.command.construct.PayloadElement;
import org.chulk.codegen.command.construct.StringLiteral;

import com.google.common.base.Joiner;
import com.google.common.collect.Lists;

public class JavaCodeMapper {

	private final Command command;

	public JavaCodeMapper(Command command) {
		this.command = command;
	}

	public String map(List<String> args, Queue<PayloadElement> payloadElements) {
		if (command.equals(Command.file)) {
			return String.format("final %s %s = FileCreator.create(\"%s\", %s);", "GeneratedFile", args.get(0), args.get(1) ,buildFileCommandJavaCode(payloadElements));
		}
		
		if (command.equals(Command.out)) {
			return String.format("%s.out(%s);", args.get(0), buildFileCommandJavaCode(payloadElements));
		}
		
		if (command.equals(Command.begin)) {
			return "public class Generator {\n"; //TODO add resource management code..
		}
		
		if (command.equals(Command.end)) {
			return "}\n";
		}
		
		if (command.equals(Command.method)) {
			return String.format("public void generate(%s) {\n", payloadElements.iterator().next().getValue());
		}

		throw new RuntimeException("Non-supported command type.");
	}

	private String buildFileCommandJavaCode(Queue<PayloadElement> payloadElements) {
		List<String> strs = Lists.newArrayList();

		for (PayloadElement element : payloadElements) {
			if (element instanceof StringLiteral) {
				strs.add("\"" + element.getValue() + "\"");
			} else if (element instanceof ModelValue) {
				strs.add(element.getValue());
			}
		}

		return Joiner.on("+").join(strs);
	}

}
